package com.herdt.javxml.kap05;

import org.xml.sax.*;
import org.xml.sax.helpers.*;
import java.io.*;

public class SAX5 extends DefaultHandler
{
  XMLReader xml_reader;

  public SAX5()
  {
    try
    {
      xml_reader = XMLReaderFactory.createXMLReader("org.apache.xerces.parsers.SAXParser");
      testFeatures();
      xml_reader.setFeature("http://xml.org/sax/features/validation", true);
      xml_reader.setFeature("http://apache.org/xml/features/validation/schema", true);
      xml_reader.setFeature("http://apache.org/xml/features/validation/dynamic", true);
      System.out.println("--- Features gesetzt ---");
      testFeatures();
      xml_reader.setContentHandler(this);
      xml_reader.setErrorHandler(this);
      InputSource is = new InputSource("com\\herdt\\javxml\\kap05\\Buecher3.xml");
      xml_reader.parse(is);
    }
    catch(SAXException saxe)
    {
      System.out.println("!  ---  Fehler beim Parsen  ---  !");
    }
    catch(IOException ioe)
    {
      System.out.println("E/A-Fehler: " + ioe.getMessage());
    }
  }

  private void testFeatures()
  {
    try
    {
      System.out.println("http://xml.org/sax/features/validation = " + xml_reader.getFeature("http://xml.org/sax/features/validation"));
      System.out.println("http://apache.org/xml/features/validation/schema = " + xml_reader.getFeature("http://apache.org/xml/features/validation/schema"));
      System.out.println("http://apache.org/xml/features/validation/dynamic = " + xml_reader.getFeature("http://apache.org/xml/features/validation/dynamic") + "\n");
    }
    catch(SAXNotRecognizedException saxEx)
    {
      System.out.println("unbekannte Feature/Eigenschaft: " + saxEx.getMessage());
    }
    catch(SAXNotSupportedException saxEx)
    {
      System.out.println("Feature/Eigenschaft wird nicht unterstuetzt: " + saxEx.getMessage());
    }
  }

  public static void main(String[] args)
  {
    new SAX5();
  }

   // ContentHandler
  int tab = 0;
  private boolean hatZeichen = false;
  private Locator loc;
  
  public void setDocumentLocator(Locator locator)
  {
    loc = locator;
    System.out.println("Locator gesetzt");
  }
  public void startPrefixMapping(String prefix, String uri) throws SAXException
  {
    System.out.println("Namensraum " + prefix + " fuer die URI " + uri + " definiert");
  }
  public void endPrefixMapping(String prefix) throws SAXException
  {
    System.out.println("\n\nEnde des Namensraums " + prefix);
  }
  public void processingInstruction(String target, String data) throws SAXException
  {
    System.out.println("Processing Instruction: " + target + ": " + data);
  }
  public void startDocument() throws SAXException
  {
    System.out.println("----  XML-Dokument Beginn  ----");
  }
  public void endDocument() throws SAXException
  {
    System.out.println("----  XML-Dokument Ende  ----");
  }
  public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException
  {
    if(hatZeichen)
      hatZeichen = false;
    else
      System.out.println();
  
    System.out.print("Zeile " + loc.getLineNumber() + ": ");
    for(int j = 0; j < tab; j++)
      System.out.print("  ");
    
    System.out.print(namespaceURI + localName);
    for(int i = 0; i < atts.getLength(); i++)
      System.out.print("  " + atts.getLocalName(i) + " = " + atts.getValue(i));
    tab++;
  }
  public void endElement(String namespaceURI, String localName, String qName) throws SAXException
  {
    tab--;
  }
  public void characters(char[] ch, int start, int length) throws SAXException
  {
    String s = (new String(ch, start, length)).trim();
    if(s.length() > 0)
    {
      System.out.println("   " + s);
      hatZeichen = true;
    }
  }

  //ErrorHandler
  public void warning(SAXParseException ex)
  {
    System.out.println("\n! Warnung: " + ex.getSystemId() +
                       "; Zeile " + ex.getLineNumber() + ": \n" + ex.getMessage());
  }
  public void error(SAXParseException ex)
  {
    System.out.println("\n! ! Fehler: " + ex.getSystemId() +
                       "; Zeile " + ex.getLineNumber() + ": \n" + ex.getMessage());
  }
  public void fatalError(SAXParseException ex)
  {
    System.out.println("\n! ! ! schwerer Fehler: " + ex.getSystemId() +
                       "; Zeile " + ex.getLineNumber() + ": \n" + ex.getMessage());
  }
}

