package com.herdt.javxml.kap06;

import java.io.*;
import org.w3c.dom.*;
import org.xml.sax.SAXException;
import org.apache.xerces.parsers.*;
import org.apache.xml.serialize.*;

public class DOMManipulieren
{
  private Document doc;

  public static void main(String[] args)
  {
    DOMManipulieren dom = new DOMManipulieren();
    dom.menue();
  }

  public DOMManipulieren()
  {
    DOMParser dom = new DOMParser();
    try
    {
      dom.setFeature("http://apache.org/xml/features/dom/include-ignorable-whitespace", false);
      dom.parse("com\\herdt\\javxml\\kap06\\NeueSchriftsteller.xml");
      doc = dom.getDocument();
      System.out.println("Strukturbaum der Datei NeueSchriftsteller.xml vollstaendig aufgebaut");
    }
    catch(SAXException se)
    {
      System.out.println("Fehler beim Parsen: " + se.getMessage());
    }
    catch(IOException e)
    {
      System.out.println("E/A-Fehler: " + e.getMessage());
    }
  }

  void menue()
  {
    int menue = 49;
    try
    {
      while(menue != 48)
      {
        System.out.println("Bitte waehlen Sie:");
        System.out.println("  1 - Struktur-Baum anzeigen");
        System.out.println("  2 - Neues Werk hinzufuegen");
        System.out.println("  3 - Schriftstellerdaten aendern");
        System.out.println("  4 - Schriftsteller loeschen");
        System.out.println("  5 - in Datei speichern");
        System.out.println("  0 - Ende");
        System.out.print("Ihre Wahl: ");
        menue = System.in.read();
        System.in.skip(System.in.available());
        switch(menue)
        {
          case 48:     // ASCII-Code fr 0
            break;
          case 49:     // ASCII-Code fr 1
            anzeigen();
            break;
          case 50:     // ASCII-Code fr 2
            hinzufuegen();
            break;
          case 51:     // ASCII-Code fr 3
            aendern();
            break;
          case 52:     // ASCII-Code fr 4
            loeschen();
            break;
          case 53:     // ASCII-Code fr 5
            ausgeben();
            break;
          default:
            System.out.println("Falsche Eingabe !");
        }
      }
    }
    catch(IOException ioex)
    {}
  }

  String readString()
  {
    BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
    String name = "";
    try
    {
      name = br.readLine();
    }
    catch(IOException ioex)
    {
      System.out.println("IOException: " + ioex.getMessage());
    }
    return(name);
  }

  void anzeigen()
  {
    System.out.println("\nStrukturbaum-Anzeige");
    System.out.println("====================\n");
    OutputFormat format = new OutputFormat(doc);
    format.setIndenting(true);
    format.setIndent(2);
    format.setLineSeparator("\r\n");
    try
    {
      XMLSerializer serializer = new XMLSerializer(System.out, format);
      serializer.serialize(doc);
    }
    catch(IOException ioex)
    {
      System.out.println("IOException: " + ioex.getMessage());
    }
    System.out.println();
  }

  Node knotenSuchen(String name)
  {
    NodeList nl = doc.getElementsByTagName("Name");

    for(int i = 0; i < nl.getLength(); i++)
    {
      if(nl.item(i).getFirstChild().getNodeValue().equals(name))
         return(nl.item(i));
      else
        System.out.println(nl.item(i).getFirstChild().getNodeValue());
    }

    System.out.println("Schriftsteller nicht gefunden\n");
    return(null);
  }

  void hinzufuegen()
  {
    System.out.println("\nneues Werk aufnehmen");
    System.out.println("====================\n");
    System.out.print("Name des Schriftstellers: ");
    String name = readString();

    DOM3 dom3 = new DOM3(1);
    Node nodeName = knotenSuchen(name);

    if(nodeName != null)
    {
      Node nodeSchriftsteller = nodeName.getParentNode();
      System.out.print("welches Werk aufnehmen: ");
      String werk = readString();
      Element nodeWerk = doc.createElement("Werk");
      Text text = doc.createTextNode(werk);
      nodeWerk.appendChild(text);
      System.out.print("Art des Werkes (z.B. Roman): ");
      String art = readString();
      nodeWerk.setAttribute("Art", art);
      System.out.print("Erscheinungsjahr: ");
      String jahr = readString();
      nodeWerk.setAttribute("Erscheinungsjahr", jahr);
      nodeSchriftsteller.appendChild(nodeWerk);
      dom3.anzeigen(nodeSchriftsteller, 1);
    }
  }

  void aendern()
  {
    System.out.println("\nAendern der Schriftstellerdaten");
    System.out.println("==============================\n");
    System.out.print("Name des Schriftstellers: ");
    String name = readString();
    DOM3 dom3 = new DOM3(1);
    Node nodeName = knotenSuchen(name);
    
    if(nodeName != null)
    {
      Node nodeCopy = nodeName.cloneNode(true);
      System.out.print("anderer Name: ");
      String name2 = readString();
      nodeCopy.getFirstChild().setNodeValue(name2);
      nodeName.getParentNode().replaceChild(nodeCopy, nodeName);
      System.out.println("Name des Schriftstellers " + name + " wurde ersetzt durch " + name2);
    }
  }

  void loeschen()
  {
    System.out.println("\nLoeschen eines Schriftstellers");
    System.out.println("==============================\n");
    System.out.print("Name des Schriftstellers: ");
    String name = readString();
    DOM3 dom3 = new DOM3(1);
    Node nodeName = knotenSuchen(name);
    if(nodeName != null)
    {
      Node nodeSchriftsteller = nodeName.getParentNode();
      dom3.anzeigen(nodeSchriftsteller, 1);
      System.out.print("wirklich loeschen (j/n): ");
      String s = readString();
      if(s.equals("j") || s.equals("J"))
      {
        nodeSchriftsteller.getParentNode().removeChild(nodeSchriftsteller);
        System.out.println("Schriftsteller " + name + " geloescht");
      }
    }
  }

  void ausgeben()
  {
  try
    {
      FileWriter fw = new FileWriter("com\\herdt\\javxml\\kap06\\NeueSchriftsteller.xml", false);
      OutputFormat format = new OutputFormat(doc);
      format.setIndenting(true);
      format.setIndent(2);
      format.setLineSeparator("\r\n");
      format.setEncoding("ISO-8859-1");
      XMLSerializer serializer = new XMLSerializer(fw, format);
      serializer.serialize(doc);
      System.out.println("XML-Dokument erfolgreich erstellt.");
    }
    catch(IOException ioex)
    {
      System.out.println("IOException: " + ioex.getMessage());
    }
  }
}
